package notify

import (
	. "context"
	"os"
	"path/filepath"
	"time"
)

// Monitor directory changes and write them on the returned channel.
func Monitor(ctx Context, dir string, freq time.Duration) chan string {
	out := make(chan string)
	// Watch for changes and report them on the channel
	visit := visitor(out)
	go func() {
		for {
			select {
			case <-ctx.Done():
				return
			default:
				filepath.Walk(dir, visit)
				time.Sleep(freq)
			}
		}
	}()
	return out
}

// visitor returns func reporting on the chan when a file was
// modified.  Modification time is compared to the last reported file.
func visitor(out chan string) filepath.WalkFunc {
	last := time.Now()
	return func(path string, f os.FileInfo, err error) error {
		if err != nil || f.IsDir() {
			return nil
		}
		if f.ModTime().After(last) {
			out <- path
			last = f.ModTime()
		}
		return nil
	}
}
