package notify_test

import (
	"context"
	"go-learn/ex/notify"
	"io/ioutil"
	"os"
	"testing"
	"time"
)

func Test_react_to_file_changes(t *testing.T) {
	freq := time.Second
	timeout := 2 * freq
	ctx, cancel := context.WithTimeout(context.Background(), timeout)
	defer cancel()
	changed := notify.Monitor(ctx, ".", freq)

	// Prepare file to be created
	file := "testfile.txt"
	os.RemoveAll(file)
	defer os.RemoveAll(file)
	// Comment to show failure, explained in timing_test.go
	time.Sleep(30 * time.Millisecond)

	// Trigger change
	ioutil.WriteFile(file, []byte("test"), 0644)
	select {
	case got := <-changed:
		exp := file
		if got != exp {
			format := "Expected %q, Monitor() signaled %q on channel"
			t.Errorf(format, exp, got)
		}
	case <-ctx.Done():
		t.Error("Monitor timeout without reaction")
	}
}
